import configparser
import os
import webbrowser

import click
import simple_term_menu
from zen_core.handlers.git_client import GitClient
from zen_core.handlers.git_repo import GitRepo

QUERY_CONFIG_FILE_PATH = os.path.expanduser('~/.config/sentry/query.ini')


def read_query():
    config_parser = configparser.ConfigParser()
    config_parser.read(QUERY_CONFIG_FILE_PATH, encoding='utf-8')
    queries = [(name, query) for name, query in config_parser['queries'].items()]
    return queries

@click.group()
@click.pass_context
def prs(ctx):
    pass


@prs.command()
@click.option('-q', '--query')
@click.pass_context
def search(ctx, query=None):
    git: GitClient = ctx.obj.get('client')
    if query is not None:
        pass
    else:
        predefined_queries = read_query()
        queries = [f'{name} - {query}' for name, query in predefined_queries]
        queries.append('Exit')
        top_term = simple_term_menu.TerminalMenu(queries)
        terminal_menu_exit = False
        while not terminal_menu_exit:
            os.system('clear')
            query_index = top_term.show()
            # query_index = simple_term_menu.TerminalMenu([f'{name} - {query}' for name, query in predefined_queries]).show()
            if queries[query_index] == 'Exit':
                terminal_menu_exit = True
                continue
            query_name, query_to_run = predefined_queries[query_index]
            issues = git._git_client.search_issues(query_to_run)
            repositories = {}
            for i in issues:
                pr = i.issue.pull_request()
                if pr is not None:
                    repository = GitRepo(pr.repository)
                    if repository.full_name() not in repositories:
                        repositories[repository.full_name()] = []
                    repositories[repository.full_name()].append(pr)
            repo_choices = list(repositories.keys())
            repo_choices.append('Back')
            repo_feed = simple_term_menu.TerminalMenu(repo_choices)
            feed_menu_back = False
            while not feed_menu_back:
                os.system('clear')
                repo_index = repo_feed.show()
                if repo_choices[repo_index] == 'Back':
                    feed_menu_back = True
                    continue
                repo = list(repositories.keys())[repo_index]
                open_prs = repositories[repo]
                choices = []
                for pr in open_prs:
                    description = f'#{pr.number} {pr.title} by {pr.user} {len(list(pr.review_comments()))} ✉'
                    state = any(r.state != 'APPROVED' for r in pr.reviews())
                    if state:
                        description = f'⨯ {description}'
                    else:
                        description = f'✓ {description}'
                    choices.append(description)
                choices.append('Back')
                feed_pr_back = False
                while not feed_pr_back:
                    pr_index = simple_term_menu.TerminalMenu(choices).show()
                    if choices[pr_index] == 'Back':
                        feed_pr_back = True
                        continue
                    p = open_prs[pr_index]
                    webbrowser.open(p.html_url)



# /usr/local/bin/python3

import os
import click
import re
import subprocess

from github3 import login
from click import pass_context
from zen_core.handlers.git_client import GitClient

from zen.commands.prs import prs
from zen.commands.repos import repos


@click.group()
@pass_context
def zen_cli(ctx):
    pass


#
# # TODO: Add config.ini support later on
# # TODO: Add workspace env variable for cloning
# # TODO: Improve testability.
# # TODO: Consider Cucumber tests for python
# @zen_cli.command()
# @click.argument('username', default=None, required=False)
# @click.option('-p', '--patterns', multiple=True, default=['.*'])
# @pass_context
# def repos(ctx, username, patterns):
#
#     git = login(token=os.environ['ZEN'])
#     if not username:
#         username = git.me().login
#         repositories = git.repositories()
#     else:
#         repositories = git.repositories_by(username)
#
#     filter_by = '|'.join([r'(\S*{}\S*)'.format(p) for p in patterns])
#     regex = re.compile(filter_by)
#
#     repositories = [r for r in repositories if regex.findall(r.name)]
#
#     if ctx.parent.parent is None:
#         click.echo(f'{username} has access to the following repositories:')
#
#         for repository in repositories:
#             click.echo(f'{repository.name}')
#
#     return repositories
#
#
# @zen_cli.command()
# @click.argument('username', default=None, required=False)
# @click.option('-p', '--patterns',  default=['.*'], multiple=True)
# @pass_context
# def clone(ctx, username, patterns):
#
#     repositories = ctx.forward(repos)
#
#     for repo in repositories:
#         _repository_clone(repo)
#
#     return repositories
#
#
# def _repository_clone(repository, path=os.environ.get('WORKSPACE')):
#     repo_dir = os.path.join(path, repository.name)
#     if os.path.isdir(repo_dir):
#         click.echo(f'{repo_dir} already exists')
#         return False
#
#     subprocess.run(['git', 'clone', repository.ssh_url, repo_dir])
#     return True


def main():
    git = GitClient()
    zen_cli.add_command(repos)
    zen_cli.add_command(prs)
    zen_cli(obj={'client': git})


if __name__ == '__main__':
    main()



import pprint

import click
import fuzzyfinder
from simple_term_menu import TerminalMenu
from zen_core.handlers.git_client import GitClient
from zen_core.handlers.git_repo import GitRepo


@click.group()
@click.pass_context
def repos(ctx):
    git: GitClient = ctx.obj.get('client')
    if git._repos is None:
        git._repos = [GitRepo(repo) for repo in git._git_client.repositories()]
    ctx.obj['client'] = git


@repos.command()
@click.argument('query', required=False)
@click.pass_context
def show(ctx, query):
    git: GitClient = ctx.obj['client']
    print(f'Welcome back {git.me().name()}')
    if query is not None:
        results = list(fuzzyfinder.main.fuzzyfinder(query, [repo.full_name() for repo in git._repos]))
    else:
        results = [repo.full_name() for repo in git._repos]
    pprint.pprint(results)


@repos.command()
@click.argument('query', required=False)
@click.pass_context
def clone(ctx, query):
    git: GitClient = ctx.obj['client']
    print(f'Welcome back {git.me().name()}')
    if query is not None:
        results = list(
            fuzzyfinder.main.fuzzyfinder(query, [repo for repo in git._repos], accessor=lambda repo: repo.full_name()))
    else:
        results = git._repos

    terminal_menu = TerminalMenu([r.full_name() for r in results],
                                 multi_select=True,
                                 show_multi_select_hint=True)
    menu_entry_indices = terminal_menu.show()
    repos = [results[index] for index in menu_entry_indices]
    for repo in repos:
        print(repo.ssh_url())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zen', 'zen.commands', 'zen.main']

package_data = \
{'': ['*']}

install_requires = \
['fuzzyfinder>=2.1.0,<3.0.0',
 'fuzzysearch>=0.7.3,<0.8.0',
 'simple-term-menu>=1.4.1,<2.0.0',
 'zen-core>=0.2.0,<0.3.0']

entry_points = \
{'console_scripts': ['zen = zen.main.zen:main']}

setup_kwargs = {
    'name': 'zen-git',
    'version': '0.1.7',
    'description': 'A git CLI utility tool',
    'long_description': '# Github Zen utility tools\n\nThis is a simple project aimed at improving command-line developer workflow with git.\n',
    'author': 'Dragos Dumitrache',
    'author_email': 'dragos@afterburner.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
